@@ -3,7 +3,7 @@ from typing import Iterable, Union, Mapping
 
 from flask_restful import Resource, fields, marshal
 
-from metadata_service.proxy import neo4j_proxy
+from metadata_service.proxy import get_proxy_client
 
 popular_table_fields = {
     'database': fields.String,
@@ -23,8 +23,8 @@ class PopularTablesAPI(Resource):
     PopularTables API
     """
     def __init__(self) -> None:
-        self.neo4j = neo4j_proxy.get_neo4j()
+        self.client = get_proxy_client()
 
     def get(self) -> Iterable[Union[Mapping, int, None]]:
-        popular_tables = self.neo4j.get_popular_tables()
+        popular_tables = self.client.get_popular_tables()
         return marshal({'popular_tables': popular_tables}, popular_tables_fields), HTTPStatus.OK
