@@ -1,24 +1,23 @@
 import logging
 import textwrap
-import time
 from random import randint
-from threading import Lock
 from typing import Dict, Any, no_type_check, List, Tuple, Union  # noqa: F401
 
+import time
 from beaker.cache import CacheManager
 from beaker.util import parse_cache_config_options
-from flask import current_app
 from neo4j.v1 import BoltStatementResult
 from neo4j.v1 import GraphDatabase, Driver  # noqa: F401
 
-from metadata_service import config
 from metadata_service.entity.popular_table import PopularTable
 from metadata_service.entity.table_detail import Application, Column, Reader, Source, \
     Statistics, Table, Tag, User, Watermark
 from metadata_service.entity.tag_detail import TagDetail
 from metadata_service.entity.user_detail import User as UserEntity
 from metadata_service.exception import NotFoundException
+from metadata_service.proxy.base_proxy import BaseProxy
 from metadata_service.proxy.statsd_utilities import timer_with_counter
+from metadata_service.util import UserResourceRel
 
 _CACHE = CacheManager(**parse_cache_config_options({'cache.type': 'memory'}))
 
@@ -29,15 +28,16 @@ _GET_POPULAR_TABLE_CACHE_EXPIRY_SEC = 11 * 60 * 60 + randint(0, 3600)
 LOGGER = logging.getLogger(__name__)
 
 
-class Neo4jProxy:
+class Neo4jProxy(BaseProxy):
     """
     A proxy to Neo4j (Gateway to Neo4j)
     """
 
     def __init__(self, *,
-                 endpoint: str,
-                 neo4j_user: str ='neo4j',
-                 neo4j_password: str ='',
+                 host: str,
+                 port: int,
+                 user: str ='neo4j',
+                 password: str ='',
                  num_conns: int =50,
                  max_connection_lifetime_sec: int =100) -> None:
         """
@@ -50,10 +50,11 @@ class Neo4jProxy:
         words, connection life time longer than this value won't be reused and closed on garbage collection. This
         value needs to be smaller than surrounding network environment's timeout.
         """
+        endpoint = f'{host}:{port}'
         self._driver = GraphDatabase.driver(endpoint, max_connection_pool_size=num_conns,
                                             connection_timeout=10,
                                             max_connection_lifetime=max_connection_lifetime_sec,
-                                            auth=(neo4j_user, neo4j_password))  # type: Driver
+                                            auth=(user, password))  # type: Driver
 
     @timer_with_counter
     def get_table(self, *, table_uri: str) -> Table:
@@ -605,7 +606,7 @@ class Neo4jProxy:
         return results
 
     @timer_with_counter
-    def get_neo4j_latest_updated_ts(self) -> int:
+    def get_latest_updated_ts(self) -> int:
         """
         API method to fetch last updated / index timestamp for neo4j, es
 
@@ -693,7 +694,9 @@ class Neo4jProxy:
         """
 
         query = textwrap.dedent("""
-        MATCH (user:User {key: $user_id}) RETURN user as user_record
+        MATCH (user:User {key: $user_id})
+        OPTIONAL MATCH (user)-[:manage_by]->(manager:User)
+        RETURN user as user_record, manager as manager_record
         """)
 
         record = self._execute_cypher_query(statement=query,
@@ -701,7 +704,13 @@ class Neo4jProxy:
         if not record:
             raise NotFoundException('User {user_id} '
                                     'not found in the graph'.format(user_id=user_id))
-        record = record.single()['user_record']
+        single_result = record.single()
+        record = single_result.get('user_record', {})
+        manager_record = single_result.get('manager_record', {})
+        if manager_record:
+            manager_name = manager_record.get('full_name', '')
+        else:
+            manager_name = ''
         result = UserEntity(email=record['email'],
                             first_name=record.get('first_name'),
                             last_name=record.get('last_name'),
@@ -710,33 +719,146 @@ class Neo4jProxy:
                             github_username=record.get('github_username'),
                             team_name=record.get('team_name'),
                             slack_id=record.get('slack_id'),
-                            employee_type=record.get('employee_type'))
+                            employee_type=record.get('employee_type'),
+                            manager_fullname=manager_name)
         return result
 
+    @staticmethod
+    def _get_relation_by_type(relation_type: UserResourceRel) -> Tuple:
 
-_neo4j_proxy = None
-_neo4j_lock = Lock()
-
-
-def get_neo4j() -> Neo4jProxy:
-    """
-    Provides singleton neo4j proxy
-    :return: Neo4j proxy
-    """
-    global _neo4j_proxy
-    endpoint = current_app.config[config.NEO4J_ENDPOINT_KEY]
-    neo4j_user = current_app.config[config.NEO4J_USER]
-    neo4j_password = current_app.config[config.NEO4J_PASSWORD]
-
-    if _neo4j_proxy:
-        return _neo4j_proxy
-
-    with _neo4j_lock:
-        if _neo4j_proxy:
-            return _neo4j_proxy
+        if relation_type == UserResourceRel.follow:
+            relation, reverse_relation = 'FOLLOW', 'FOLLOWED_BY'
+        elif relation_type == UserResourceRel.own:
+            relation, reverse_relation = 'OWNER_OF', 'OWNER'
+        elif relation_type == UserResourceRel.read:
+            relation, reverse_relation = 'READ', 'READ_BY'
         else:
-            _neo4j_proxy = Neo4jProxy(endpoint=endpoint,
-                                      neo4j_user=neo4j_user,
-                                      neo4j_password=neo4j_password)
+            raise NotImplementedError('The relation type {} is not defined!'.format(relation_type))
+        return relation, reverse_relation
 
-    return _neo4j_proxy
+    @timer_with_counter
+    def get_table_by_user_relation(self, *, user_email: str, relation_type: UserResourceRel) -> Dict[str, Any]:
+        """
+        Retrive all follow the resources per user based on the relation.
+        We start with table resources only, then add dashboard.
+
+        :param user_email: the email of the user
+        :param relation_type: the relation between the user and the resource
+        :return:
+        """
+        relation, _ = self._get_relation_by_type(relation_type)
+        # relationship can't be parameterized
+        query_key = 'key: "{user_id}"'.format(user_id=user_email)
+
+        query = textwrap.dedent("""
+        MATCH (user:User {{{key}}})-[:{relation}]->(tbl:Table)
+        RETURN COLLECT(DISTINCT tbl) as table_records
+        """).format(key=query_key,
+                    relation=relation)
+
+        record = self._execute_cypher_query(statement=query,
+                                            param_dict={})
+
+        if not record:
+            raise NotFoundException('User {user_id} does not {relation} '
+                                    'any resources'.format(user_id=user_email,
+                                                           relation=relation))
+        results = []
+        table_records = record.single().get('table_records', [])
+
+        for record in table_records:
+            # todo: decide whether we want to return a list of table entities or just table_uri
+            results.append(self.get_table(table_uri=record['key']))
+        return {'table': results}
+
+    @timer_with_counter
+    def add_table_relation_by_user(self, *,
+                                   table_uri: str,
+                                   user_email: str,
+                                   relation_type: UserResourceRel) -> None:
+        """
+        Update table user informations.
+        1. Do a upsert of the user node.
+        2. Do a upsert of the relation/reverse-relation edge.
+
+        :param table_uri:
+        :param user_email:
+        :param relation_type:
+        :return:
+        """
+        relation, reverse_relation = self._get_relation_by_type(relation_type)
+
+        upsert_user_query = textwrap.dedent("""
+        MERGE (u:User {key: $user_email})
+        on CREATE SET u={email: $user_email, key: $user_email}
+        on MATCH SET u={email: $user_email, key: $user_email}
+        """)
+
+        user_email = 'key: "{user_email}"'.format(user_email=user_email)
+        tbl_key = 'key: "{tbl_key}"'.format(tbl_key=table_uri)
+
+        upsert_user_relation_query = textwrap.dedent("""
+        MATCH (n1:User {{{user_email}}}), (n2:Table {{{tbl_key}}})
+        MERGE (n1)-[r1:{relation}]->(n2)-[r2:{reverse_relation}]->(n1)
+        RETURN n1.key, n2.key
+        """).format(user_email=user_email,
+                    tbl_key=tbl_key,
+                    relation=relation,
+                    reverse_relation=reverse_relation)
+
+        try:
+            tx = self._driver.session().begin_transaction()
+            # upsert the node
+            tx.run(upsert_user_query, {'user_email': user_email})
+            result = tx.run(upsert_user_relation_query, {})
+
+            if not result.single():
+                raise RuntimeError('Failed to create relation between '
+                                   'user {user} and table {tbl}'.format(user=user_email,
+                                                                        tbl=table_uri))
+            tx.commit()
+        except Exception as e:
+            if not tx.closed():
+                tx.rollback()
+            # propagate the exception back to api
+            raise e
+        finally:
+            tx.close()
+
+    @timer_with_counter
+    def delete_table_relation_by_user(self, *,
+                                      table_uri: str,
+                                      user_email: str,
+                                      relation_type: UserResourceRel) -> None:
+        """
+        Delete the relationship between user and resources.
+
+        :param table_uri:
+        :param user_email:
+        :param relation_type:
+        :return:
+        """
+        relation, reverse_relation = self._get_relation_by_type(relation_type)
+
+        user_email = 'key: "{user_email}"'.format(user_email=user_email)
+        tbl_key = 'key: "{tbl_key}"'.format(tbl_key=table_uri)
+
+        delete_query = textwrap.dedent("""
+        MATCH (n1:User {{{user_email}}})-[r1:{relation}]->
+        (n2:Table {{{tbl_key}}})-[r2:{reverse_relation}]->(n1) DELETE r1,r2
+        """).format(user_email=user_email,
+                    tbl_key=tbl_key,
+                    relation=relation,
+                    reverse_relation=reverse_relation)
+
+        try:
+            tx = self._driver.session().begin_transaction()
+            tx.run(delete_query, {})
+            tx.commit()
+        except Exception as e:
+            # propagate the exception back to api
+            if not tx.closed():
+                tx.rollback()
+            raise e
+        finally:
+            tx.close()
